unit UnitAutoBackup;


interface

uses System.zip, Generics.Collections, System.Classes;

type TAutoBackup = class(TObject)
    private
        fZip : TZipFile;
        fClosed : boolean;
        fErrors : TList<String>;
        function GetNewStream(filename : string) : TFileStream;
        class procedure BackupCanClose(Sender: TObject; var CanClose: Boolean);
        class function TestArchive(fullFilename : string) : boolean;
    public
        constructor Create(BackupFullName : string);
        destructor Destroy; override;
        procedure Close;

        class function GetBackupName(programInitials : string) : string;
        function AddToBackup(DataPath : string) : boolean;
        property Errors : TList<String> read fErrors;

        class function DialogBackup(WinHandle : THandle; ProgramInitials : string; DialogPath : string; backupPaths : TList<String>) : string;
end;


implementation

uses WIndows, messages, Forms, SysUtils,  System.IOUtils, Dialogs, UnitCRC32;

var major, minor : integer;
var build : string;

constructor TAutoBackup.Create(BackupFullName : string);
begin
    fzip := TZipFile.Create;
    fzip.Open(BackupFullName, zmWrite);
    ferrors := TList<String>.create;
end;
destructor TAutoBackup.Destroy;
begin
    FreeAndNil(ferrors);
    FreeAndNil(fzip);
    FreeAndNil(ferrors);
end;
procedure TAutoBackup.Close;
begin
    fzip.Close;
    //ferrors.Clear;
end;

class function TAutoBackup.DialogBackup(WinHandle : THandle; ProgramInitials : string; DialogPath : string; backupPaths : TList<String>) : string;
var s : string;
    ab : TAutoBackup;
    backupFullName, errs : string;
    sdBackup : TSaveDialog;
    i : integer;
    works : boolean;
begin
    //TAutoBackup.TestArchive('E:\_My Documents\Quick Share\AC Debugging\test.zip');

    sdbackup := TSaveDialog.Create(nil);

    sdBackup.Options := sdBackup.Options + [ofPathMustExist];
    sdBackup.Filter := 'Zip (*.zip)|*.zip';
    sdBackup.DefaultExt := 'zip';

    sdBackup.InitialDir := DialogPath;
    sdBackup.FileName := TAutoBackup.GetBackupName(ProgramInitials);
    sdBackup.OnCanClose := TAutoBackup.BackupCanClose;
    if sdBackup.Execute(WinHandle) then begin
        if FileExists(sdBackup.Files[0]) then begin
            ShowMessage('File already exists. Backup not created.');
        end else begin
            backupFullName := sdBackup.Files[0];
            ab := TAutoBackup.Create(backupFullName);
            works := true;
            errs := '';

            for s in backupPaths do begin
                works := ab.AddToBackup( s );
                if not works then BREAK;
            end;
            ab.Close;

            if not works then begin
                for i := 0 to ab.fErrors.Count-1 do begin
                    errs := errs + ab.fErrors[i] + #13#10;
                end;
                result := 'Backup Error - error adding file the ZIP archive'+#13#10+#13#10+errs;
                ShowMessage(result);
            end else if TAutoBackup.TestArchive(backupFullname) then begin
                result := '';
                ShowMessage('Backup Complete'+#13#10+backupFullName);
            end else begin
                for i := 0 to ab.fErrors.Count-1 do begin
                    errs := errs + ab.fErrors[i] + #13#10;
                end;
                result := 'Backup Error - file in ZIP archive corrupted'+#13#10+#13#10+errs;
                ShowMessage(result);
            end;

            FreeAndNil(ab);
        end;
    end;

    FreeAndNil(sdBackup)
end;
class function TAutoBackup.TestArchive(fullFilename : string) : boolean;
var
    zf : TZipFile;
    zh : TZipHeader;
    ms : TMemoryStream;
    i : integer;
    c : cardinal;
    err : boolean;
begin
    result := false;
    err := false;
    try

        zf := TZipFile.Create;
        zf.Open(fullFilename, zmRead);

        for i := 0 to zf.FileCount-1 do begin
            ms := TMemoryStream.Create;
            zf.Read(i, TStream(ms), zh);
            CRC32.ReInit;
            ms.Position := 0;
            c := not CRC32.Update(ms);
            FreeAndNil(ms);
            if c <> zh.CRC32 then begin
                err := true;
                BREAK;
            end;
        end;

        zf.Close;
        FreeAndNil(zf);

        result := true and (not err);
    except on e: Exception do
        result := false;
    end;
end;
class procedure TAutoBackup.BackupCanClose(Sender: TObject; var CanClose: Boolean);
var sd : TSaveDialog;
begin
    sd := TSaveDialog(Sender);


    CanClose := (not FileExists(sd.Files[0]));

    if not CanClose then begin
        ShowMessage('File already exists.');
    end;
end;
class function TAutoBackup.GetBackupName(programInitials : string) : string;
begin
    result := programInitials + 'v'+IntToStr(major);
    if minor < 10 then begin
        result := result + '0';
    end;
    result := result + IntToStr(minor) + ' '+build+'.zip';
end;

function TAutoBackup.AddToBackup(DataPath : string) : boolean;
var startingFolder : string;
    function ScanAndZip(path : string) : boolean;
    var s, filename, subfolder : string;
        fa : TFileAttributes;
        err : boolean;
        fs : TFileStream;
    begin
        err := false;
        for s in TDirectory.GetFileSystemEntries(path) do begin
            filename := TPath.GetFileName(s);
            fa := TPath.GetAttributes(s);
            if TFileAttribute.faDirectory in fa then begin
                if not ((filename = '.') or (filename = '..')) then begin
                    result := ScanAndZip( IncludeTrailingPathDelimiter(s) );
                    err := not result;
                end;
            end else begin
                if TPath.GetExtension(s) = '.zip' then CONTINUE;
                try
                    subfolder := TPath.GetDirectoryName(s);
                    Delete(subfolder, 1, length(startingFolder));
                    err := false;
                    if subfolder <> '' then begin
                        fZip.add(s, IncludeTrailingPathDelimiter(subfolder) + filename);
                    end else begin
                        fZip.Add(s);
                    end;
                except on e: Exception do
                    begin
                        ferrors.Add(E.message);
                        err := true;
                        try
                            fs := Self.GetNewStream(s);
                            if subfolder <> '' then begin
                                fZip.add(fs, IncludeTrailingPathDelimiter(subfolder) + filename);
                            end else begin
                                fZip.Add(fs, filename);
                            end;
                            FreeAndNil(fs);
                            err := false;
                        except
                            ferrors.Add(E.message);
                            result := false;
                        end;
                    end;
                end;
            end;

            if (err) then begin
                BREAK;
            end;
        end;
        result := not err;
    end;
begin
    startingFolder := IncludeTrailingPathDelimiter(DataPath);
    result := ScanAndZip(startingFolder);
end;
function TAutoBackup.GetNewStream(filename : string) : TFileStream;
begin
    result := TFileStream.Create(filename, fmShareDenyNone or fmOpenRead);
end;

function getAppFolder : string;
begin
    result := IncludeTrailingPathDelimiter(
        TPath.GetDirectoryName(Application.ExeName)
    );
end;
procedure GetAppVersion(var major, minor : integer; var build : string);
var L, i : DWORD;
    p, buf : Pointer;
    ver : array [0 .. 3] of word;
begin
    L := Windows.GetFileVersionInfoSize(PChar(Application.EXEName), i);
    if (L = 0) then begin
        EXIT;
    end;


    GetMem(buf, L);
        Windows.GetFileVersionInfo(PChar(Application.EXEName), 0, L, Buf);
        Windows.VerQueryValue(Buf, '\', p, i);

        Ver[0] := HiWord(TVSFixedFileInfo(p^).dwFileVersionMS);
        Ver[1] := LoWord(TVSFixedFileInfo(p^).dwFileVersionMS);
        Ver[2] := HiWord(TVSFixedFileInfo(p^).dwFileVersionLS);
        Ver[3] := LoWord(TVSFixedFileInfo(p^).dwFileVersionLS);

    FreeMem(Buf);

    Major := ver[0];
    Minor := ver[1];
    Build := IntToStr(ver[2]) +'-' + IntToStr(ver[3]);
end;

initialization
begin
    GetAppVersion(major, minor, build);
end;


end.
